#
#  file: example.tcl
#  Written by Yukio Hashimoto
#  Added by Nguyen Huu Thanh
#
#  This example clarifies how to use Agent/Sar and Agent/Pass  
#
#  (CBR Sink) p5 <--p4 <--p3 <--sar packets--p2 <--p1 <--p0 (CBR Source)
#  (TCP Source) p5 -->p4 -->p3 --sar packets-->p2 -->p1 -->p0 (TCP Sink)
#
#   Node p2 does segmentation and node p3 does reassembly for CBR.
#   Node tcp_sar_n3 does segmentation and tcp_sar_n2 does reassembly for TCP.
#   


#Create a simulator object
set ns [new Simulator]

$ns color 0 blue
$ns color 1 red
$ns color 2 green

#Open a trace file
set nf [open out.nam w]
$ns namtrace-all $nf

#Define a 'finish' procedure
proc finish {} {
        global ns nf
        $ns flush-trace
        close $nf
        exec nam out.nam &
        exit 0
}

#Create three nodes
set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]
set n4 [$ns node]
set n5 [$ns node]

#Connect the nodes with two links
$ns duplex-link $n0 $n1 10Mb 10ms DropTail
$ns duplex-link $n1 $n2 10Mb 10ms DropTail
$ns duplex-link $n2 $n3 10Mb 1ms DropTail
$ns duplex-link $n3 $n4 10Mb 10ms DropTail
$ns duplex-link $n4 $n5 10Mb 10ms DropTail



##### queue limit
$ns queue-limit $n2 $n3  2000
$ns queue-limit $n3 $n2  2000

set tff [open simp.out.tr2 w]
#$ns trace-queue $n2 $n3 $tff
#$ns trace-queue $n0 $n1 $tff
$ns trace-queue $n1 $n0 $tff
$ns trace-queue $n5 $n4 $tff


set p0 [new Agent/CBR]
$p0 set packetSize_ 20000
$p0 set interval_ 0.1s
$p0 set class_ 0
$ns attach-agent $n0 $p0


set p1 [new Agent/Pass]
$ns attach-agent $n2 $p1

set tcp_pass_n2 [new Agent/Pass]
$ns attach-agent $n2 $tcp_pass_n2 

####          #### p1 -- --- p2 
set p2 [new Agent/Sar]
$ns attach-agent $n2 $p2

set tcp_sar_n2 [new Agent/Sar]
$ns attach-agent $n2 $tcp_sar_n2

####          #### p2 ---Sar --- p3 
set p3 [new Agent/Sar]
$ns attach-agent $n3 $p3

set tcp_sar_n3 [new Agent/Sar]
$ns attach-agent $n3 $tcp_sar_n3

####          #### p3 --- --- p4 
set p4 [new Agent/Pass]
$ns attach-agent $n3 $p4

set tcp_pass_n3 [new Agent/Pass]
$ns attach-agent $n3 $tcp_pass_n3

####          #### p4 ---CBR --- p5 
set p5 [new Agent/Null]
$ns attach-agent $n5 $p5

set tcp [new Agent/TCP]
$tcp set class_ 1
$ns attach-agent $n5 $tcp
set ftp [new Application/FTP]
$ftp attach-agent $tcp

set sink [new Agent/TCPSink]
$ns attach-agent $n0 $sink
#$ns connect $tcp $sink


$p1 dst $p2
$p2 dst $p1
$p3 dst $p4
$p4 dst $p3

$tcp_pass_n2 dst $tcp_sar_n2
$tcp_sar_n2 dst $tcp_pass_n2
$tcp_pass_n3 dst $tcp_sar_n3
$tcp_sar_n3 dst $tcp_pass_n3


#Connect the two agents
$ns connect $p0 $p1
$ns connect $p2 $p3
$ns connect $p4 $p5

$ns connect $tcp $tcp_pass_n3
$ns connect $tcp_sar_n3 $tcp_sar_n2
$ns connect $tcp_pass_n2 $sink

#Schedule events
$ns at 0.01 "$p0 start"
$ns at 0.2 "$ftp start"
$ns at 1.0 "finish"

#Run the simulation
$ns run


