/*
 * File: Code for a 'Sar' Agent Class
 * date: July, 1998
 * Author: Yukio Hashimoto
 * Thanks to Marc Greis (greis@cs.uni-bonn.de)
 * Modified by Nguyen Huu Thanh (thanh@informatik.unibw-muenchen.de)
 * date: June 28 1999
 */

#include <stdio.h>
#include <math.h>
#include "sar.h"
#include "rtp.h"
#include "tcp.h"
#include <iostream.h>

static class SarClass : public TclClass {
public:
  SarClass() : TclClass("Agent/Sar") {}
  TclObject* create(int, const char*const*) {
    return (new SarAgent());
  }
} class_sar;


SarAgent::SarAgent() : Agent(PT_SAR), sar_timer_(this), finish_seg(1)
 {
   bind("packetSize_", &size_);
   bind("off_sar_", &off_sar_); 
   bind("off_rtp_", &off_rtp_);
   bind("off_tcp_", &off_tcp_);
   finish_seg=1;

 }


 int SarAgent::command(int argc, const char*const* argv)
 {
   if (argc == 3) {
	if (strcmp(argv[1], "dst") == 0) {
       Agent* a = (Agent*)TclObject::lookup(argv[2]);
       dst=a;
       /*
       // Create a new packet
       Packet* pkt = allocpkt();
       // Access the Sar header for the new packet:
       hdr_sar* hdr = (hdr_sar*)pkt->access(off_sar_);
       // Set the 'ret' field to 0, so the receiving node knows
       // that it has to generate an echo packet
       hdr->ret = 0;
       // Store the current time in the 'send_time' field
       hdr->send_time = Scheduler::instance().clock();
       // Send the packet
       send(pkt, 0);
       // return TCL_OK, so the calling function knows that the
       // command has been processed
       */
       return (TCL_OK);
     }
   }
   // If the command hasn't been processed by SarAgent()::command,
   // call the command() function for the base class
   return (Agent::command(argc, argv));
 }


 void SarAgent::recv(Packet* pkt, Handler*)
 {
   // Access the IP header for the received packet:
   hdr_cmn* hdrcmn = (hdr_cmn*)pkt->access(off_cmn_);
   hdr_ip* hdrip = (hdr_ip*)pkt->access(off_ip_);
   //hdr_rtp* rh_from = (hdr_rtp*)pkt->access(off_rtp_);



   if(hdrcmn->ptype() == PT_SAR){// to upper layer 
   // reassembly

     // Access the Sar header for the received packet:
     hdr_sar* hdr = (hdr_sar*)pkt->access(off_sar_);

     if(hdr->total_cell_num == hdr->cell_num) {
       //Packet* p = allocpkt();
       //hdr_sar* hdr3 = (hdr_sar*)p->access(off_sar_);

       hdrcmn->size() = hdr->pkt_size;  // restore the original size

       //hdr3->pkt_size=hdr->pkt_size;
       //hdr3->packet_type_ = hdr->packet_type_;

       // kokowa test
       //hdr_rtp* rh = (hdr_rtp*)p->access(off_rtp_);
       //*rh= *rh_from;

       //dst->recv(p,(Handler*)0);       

       dst->recv(pkt,(Handler*)0);

     }else {
       //cout << "discard";
       Packet::free(pkt);

     }


     //Packet::free(pkt);

   } else { // from upper layer  
     // segmentation

     sar_timer_.resched(0.2);
     
     while (finish_seg == 0) {
       cout << "wait "<< endl ;
     } // do nothing


    int psize=hdrcmn->size(); // read packet size
    packet_t org_ptype_=hdrcmn->ptype();

    // fragment the packet into 48 byte payload of the ATM cell (total 53 bytes)
    int cell_num =(int)ceil((double)psize/48.0);  


    for(int i=0; i< cell_num-1; i++) {    
      Packet* c = allocpkt();

      
      hdr_sar* hdr_sar1 = (hdr_sar*)c->access(off_sar_);

      hdr_sar1->total_cell_num=cell_num;
      hdr_sar1->cell_num=i+1;

      /* put flow id  into the cells, otherwise the flow of cells in
       * the segmentation path might look awkward in nam.
       * also for flow-based packet classification */

      hdr_ip* hdr_ip1 = (hdr_ip*)c->access(off_ip_);
      int flow_ID_ = hdrip->flowid();
      hdr_ip1->flowid() = flow_ID_;

      //cout << i << "th iteration" << endl;
      send(c,0);
      
      
    }


    // last cell

    //Packet* c=allocpkt();
    //hdr_sar* hdr2 = (hdr_sar*)c->access(off_sar_);

    hdr_sar* hdr2 = (hdr_sar*)pkt->access(off_sar_);
    hdr2->total_cell_num=cell_num;
    hdr2->cell_num=cell_num;
    hdr2->IP = *hdrip;


    // save the original size and type in sar header
    hdr2->pkt_size=psize; // put IP size info
    hdr2->packet_type_=org_ptype_; // save the original packet type


    //hdr_rtp* rh = (hdr_rtp*)c->access(off_rtp_);
    //*rh = *rh_from;  // copy rtp info

    // modify the packet size and type
    hdrcmn->ptype() = type_;
    hdrcmn->size() = size_;

    // modify the ip destination
    hdrip->dst() = dst_;

    //send(c,0);
    
    send(pkt,0);
    
    //Packet::free(pkt);
  }

}


void Sar_Timer::expire(Event *e) {
  a_->timeout(0);
}


void SarAgent::timeout(int)
{
  finish_seg=1;
}

	
